#!/usr/local/bin/perl5
# regression tests against OpenSSL

use Expect;


sub openssl_execute_client {
  local($opensslkeyfile)=@_;
  $client_err="";
  $command="$OPENSSL_PATH/openssl s_client -cert $opensslkeyfile";
  &debug("Executing OpenSSL client as $command");

  ($client=Expect->spawn($command)) ||
    die("Couldn't start OpenSSL client: $!");
  
  print $client <<FOO;
GET / HTTP/1.0
User-Agent: EKR-o-matic

FOO

  while(<$client>){
    next if(/verify error/);

    if(/error/){
      print;
      $client_err=$_;
      last;
    }
  }

  $client->hard_close();
  $client_err;
}


sub openssl_execute_server { 
  local($opensslkeyfile,$client_auth)=@_;
  
  $clientauth=$client_auth?"-verify 10":"";

  $server_err="";
  $command="$OPENSSL_PATH/openssl s_server -www $clientauth -cert $opensslkeyfile";
  &debug("Executing OpenSSL server as: $command");
  ($server=Expect->spawn($command)) 
    || die("Couldn't spawn OpenSSL server");
}

sub openssl_cleanup_server {
  $server->hard_close();
  
  #be totally sure
  local($pid)=$server->pid();
  &debug("Killing $pid");
  kill(9,$server->pid());
"";
}
1; # require fodder
